import numpy as np
import os
import glob
import math
import shutil
import datetime
import fileinput
import subprocess

start_time = datetime.datetime.now()

masses = np.linspace(0.002, 0.1, 50)
masses = [format(num, '.4f') for num in masses]

model_runtimes = []

for mass in masses:
	os.system('rm -rf *Msun')				# Clear all existing directories - careful!
os.system('rm -rf Output')

for mass in masses: # Copies a new work directory to start each model
	os.system('cp -r $MESA_DIR/star/work '+str(mass)+'Msun')
	os.chdir(str(mass)+'Msun')
	os.system('./mk') # Makes the necessary files for MESA to run the model
	try:
		while subprocess.check_output(["pidof",'make']).strip(): # Delay future commands while make is running - after it is done, moves on to the next step
			pass
	except subprocess.CalledProcessError, e:
		os.system('cp ../default_inlist_project inlist_project') # Copies the default inlist_project to the current model directory
		os.system('cp ../default_inlist inlist') # Same as above, just for the inlist
		os.system('cp ../default_data_columns.list default_data_columns.list') # Same as above, just for the default data columns
		for line in fileinput.input('inlist_project', inplace=True):
			print line.replace('TEMP_MASS_CGS', str(float(mass)*1.989*(10**33))).rstrip() # cgs unit conversion
		for line in fileinput.input('inlist_project', inplace=True):
			print line.replace('TEMP_RADIUS_CGS', str(math.pow(23*float(mass), 2.0/3)*6.957*(10**10))).rstrip() # cgs unit conversion 
		os.system('./rn')
		try:
			while subprocess.check_output(["pidof",'star']).strip(): # Delay future commands while the model is running, continues after model has terminated
				pass
		except subprocess.CalledProcessError, e:
			print '~~~~~ FINISHED WITH '+str(mass)+' MSUN MODEL ~~~~~'
			os.chdir('..')
		model_runtimes.append(datetime.datetime.now())

with open('runtimes.txt', 'w') as runtime_file: # Output file which displays how long individual models took to complete (in h:m:s, not computer time)
	for mass, line in zip(masses, range(len(model_runtimes))):
		if line == 0:
			runtime_file.write(str(mass)+' MSun model took '+str(model_runtimes[line]-start_time)+' to finish\n')
		else:
			runtime_file.write(str(mass)+' MSun model took '+str(model_runtimes[line]-model_runtimes[line-1])+' to finish\n')
	runtime_file.write('Total runtime = '+str(datetime.datetime.now()-start_time)+'\n')


# Making the output directories for each individual model
try:
	os.mkdir('Output') # Main directory which contains all the model outputs
except OSError, e:
	pass
for directory in glob.glob('*Msun'): # Individual directories named by the initial mass of the model
	try:
		shutil.copyfile(directory+'/LOGS/history.data', 'Output/'+directory+'.data') # Renames the individual outputs to indicate the starting masses of each model
	except IOError, e:
		pass
with open('brown_dwarf_models.data', 'w') as outfile: # Combines all the individual outputs into one long list of continuous data
	for file in np.sort(glob.glob('Output/*.data')):
		with open(file, 'r') as infile:
			contents = infile.read()
			index = contents.index('Teff')
			outfile.write(contents[index+6:])
